#!/usr/bin/python
#
# Russell J Foster
# Argonne National Laboratory
# rjfoster@anl.gov
# rjf@russfoster.com
# Last Updated: 2019y10m02d11h
#
# ---------------------------------------------------------------------
# This script takes the output of Cisco's "show cdp neighbors" and 
# converts it to a csv formatted output.
#
# Usage:
# 
# This script assumes the devices name is located in the first part of
# the filename. i.e. [device_name].[fqdn].[extension]
#
# There are two parts to this script;
# 1) Load the cdp_nei file and parse through it.
# Sometimes data is split in two lines, sometimes it's all on one.
# So we loop through the file, find what we think is valuable, and
# put all data for one neighbor on a single line.
# 2) Loop through the lines of data, pull out the individual parts of
# data elements, print in csv format.
#

import sys
import os.path
import re
import argparse

def convert2csv ( bigline ):
	if not re.search ( "^Capability|^Device", bigline ):
		#                        1           2           3           4         5          6           7
		#                     far dev    local int     hold       caps      plat       far int
		ss1 = re.search( "^.*?(\S.*?)\s.*?(\S.*?)\s.*?(\S.*?)\s.*?(.*)\s.*?(\S.*?)\s.*?(\S.*?)\s.*?$" , bigline )
		#                 ^---(+++++)-----(+++++)-----(+++++)-----(++)-----(+++++)-----(+++++)-----$
		if ss1:
			dev1 = ss1.group(1)
			dev2 = ""
			dev3 = ""
			# If we find either a '.' or a '(' then seperate the two parts
			ss2 = re.search ( "(\S.*?)([\.\(].*)", dev1 )
			#                  (+++++)(++++++++)
			if ss2:
				dev1 = ss2.group(1)
				dev2 = ss2.group(2)
				ss3 = re.search ( "(.*)(\((.*)\))", dev2 )
				#                  (++)(++++++++)
				#                       \(    \)   <-- literal matching of parens
				if ss3:
					dev2 = ss3.group(1)
					dev3 = ss3.group(2)
			
			print '"' + local_device + '"'+sep+'"' + ss1.group(2) + '"'+sep+'"' + dev1 + '"'+sep+'"' + dev2 + '"'+sep+'"' + dev3 + '"'+sep+'"' + ss1.group(6) + '"'+sep+'"' + ss1.group(5) + '"'+sep+'"' + ss1.group(4).strip() + '"'
			if debug_level > 3: print ( "### %s" % ( bigline ) )
	else:
		if debug_level > 0: print "### ", bigline


# Grab arguments from the command line.
parser = argparse.ArgumentParser(description="This script converts the results of 'show cdp neighbor' and turns it into csv output. CSV data is sent to stdout, so redirect or pipe as needed.")
parser.add_argument('filename',help="The name of the file which contains the output of 'show cdp neighbor'")
parser.add_argument('-c',dest="comma",default=False,action="store_true",help="Seperate data elements by a comma (',') rather than a tab. Tab is default.")
parser.add_argument('-n',dest="local_dev_name",action="store",help="Set the local hostname. If this flag is NOT set we assume the first part of the filename is the local node name.")
parser.add_argument('-dv',dest="debug_level",default=0,type=int,help="Turn on debug verbosity level. '0' is off (default). This will invalidate the CSV output structure, but is useful for troubleshooting.")
parsed = parser.parse_args()

debug_level = parsed.debug_level

if debug_level > 0: print ( "### Debugging on, level %d " % ( debug_level ) )

# Results are TAB seperated by default. Using -c makes them comma seperated.

if parsed.comma:
	sep = ","		# Comma seperated values
else:
	sep = "\t"		# Tab seperated values -- this is the default

if debug_level > 0: print ( "### parsed.filename = ", parsed.filename )
if debug_level > 0: print ( "### parsed.local_dev_name = ", parsed.local_dev_name )

# Check if the file exists. 
if os.path.isfile(parsed.filename):
	# If the file exists, open it
	with open(parsed.filename) as fh1:
		# Save the contents of the file to an array
		cdp_nei = fh1.readlines()
else:
	print "File not found: ",parsed.filename
	sys.exit()

# If local device name is not specific at the command line, then extrapolate it from the filename
if parsed.local_dev_name:
	local_device = parsed.local_dev_name
else:
	ss1 = re.search( "(.*?)\.", os.path.basename(parsed.filename))
	if ss1:
		local_device = ss1.group(1)
	else:
		local_device = os.path.basename(parsed.filename)

if debug_level > 0: print ( "### local_device = ", local_device )

fulldata = ""

for singleline in cdp_nei:

	ss1 = re.search ( "^\s(.*)", singleline)
	if ss1:
		fulldata = fulldata + " " + ss1.group(1).strip()

	#ss1 = re.search ( "^(\S.*?)", singleline )
	if re.search  ( "^(\S.*?)", singleline ):
		startnext = singleline
		if fulldata != "":
			convert2csv ( fulldata )
		fulldata = startnext.strip()

convert2csv( fulldata )
